@@ -20,7 +20,7 @@
 trind.generator <- function(K=2) {
 ## Generates index arrays for 'upper triangular' storage up to order 4
 ## Suppose you fill an array using code like...
-## m = 0
+## m = 1
 ## for (i in 1:K) for (j in i:K) for (k in j:K) for (l in k:K) {
 ##   a[,m] <- something; m <- m+1 }
 ## ... and do this because actually the same 'something' would 
@@ -175,7 +175,7 @@ gamlss.etamu <- function(l1,l2,l3=NULL,l4=NULL,ig1,g2,g3=NULL,g4=NULL,i2,i3=NULL
 } # gamlss.etamu
 
 
-gamlss.gH <- function(X,jj,l1,l2,i2,l3=0,i3=0,l4=0,i4=0,d1b=0,d2b=0,deriv=0,fh=NULL,D=NULL) {
+gamlss.gH0 <- function(X,jj,l1,l2,i2,l3=0,i3=0,l4=0,i4=0,d1b=0,d2b=0,deriv=0,fh=NULL,D=NULL) {
 ## X[,jj[[i]]] is the ith model matrix.
 ## lj contains jth derivatives of the likelihood for each datum,
 ## columns are w.r.t. different combinations of parameters.
@@ -229,7 +229,7 @@ gamlss.gH <- function(X,jj,l1,l2,i2,l3=0,i3=0,l4=0,i4=0,d1b=0,d2b=0,deriv=0,fh=N
           v <- v + l3[,i3[i,i,q]] * d1eta[ind,l]
           ind <- ind + n
         }
-        d1H[jj[[j]],l] <- colSums(X[,jj[[i]],drop=FALSE]*(v*X[,jj[[i]],drop=FALSE]))
+        d1H[jj[[i]],l] <- colSums(X[,jj[[i]],drop=FALSE]*(v*X[,jj[[i]],drop=FALSE]))
       } 
     }
   } ## if deriv==1
@@ -304,6 +304,142 @@ gamlss.gH <- function(X,jj,l1,l2,i2,l3=0,i3=0,l4=0,i4=0,d1b=0,d2b=0,deriv=0,fh=N
     }
   } ## if deriv>2
 
+  list(lb=lb,lbb=lbb,d1H=d1H,d2H=d2H,trHid2H=trHid2H)
+} ## end of gamlss.gH0
+
+
+
+gamlss.gH <- function(X,jj,l1,l2,i2,l3=0,i3=0,l4=0,i4=0,d1b=0,d2b=0,deriv=0,fh=NULL,D=NULL) {
+## X[,jj[[i]]] is the ith model matrix.
+## lj contains jth derivatives of the likelihood for each datum,
+## columns are w.r.t. different combinations of parameters.
+## ij is the symmetric array indexer for the jth order derivs...
+## e.g. l4[,i4[i,j,l,m]] contains derivatives with 
+## respect to parameters indexed by i,j,l,m
+## d1b and d2b are first and second derivatives of beta w.r.t. sps.
+## fh is a factorization of the penalized hessian, while D contains the corresponding
+##    Diagonal pre-conditioning weights.
+## deriv: 0 - just grad and Hess
+##        1 - diagonal of first deriv of Hess
+##        2 - first deriv of Hess
+##        3 - everything.
+  K <- length(jj)
+  p <- ncol(X);n <- nrow(X)
+  trHid2H <- d1H <- d2H <- NULL ## defaults
+
+  ## the gradient...
+  lb <- rep(0,p)
+  for (i in 1:K) { ## first derivative loop
+    lb[jj[[i]]] <- lb[jj[[i]]] + colSums(l1[,i]*X[,jj[[i]],drop=FALSE]) ## !
+  }
+  
+  ## the Hessian...
+  lbb <- matrix(0,p,p)
+  for (i in 1:K) for (j in i:K) {
+    A <- t(X[,jj[[i]],drop=FALSE])%*%(l2[,i2[i,j]]*X[,jj[[j]],drop=FALSE])
+    lbb[jj[[i]],jj[[j]]] <- lbb[jj[[i]],jj[[j]]] + A 
+    if (j>i) lbb[jj[[j]],jj[[i]]] <- lbb[jj[[j]],jj[[i]]] + t(A) 
+  } 
+
+  if (deriv>0) {
+    ## the first derivative of the Hessian, using d1b
+    ## the first derivates of the coefficients wrt the sps
+    m <- ncol(d1b) ## number of smoothing parameters
+    ## stack the derivatives of the various linear predictors on top
+    ## of each other...
+    d1eta <- matrix(0,n*K,m)
+    ind <- 1:n
+    for (i in 1:K) { 
+      d1eta[ind,] <- X[,jj[[i]],drop=FALSE]%*%d1b[jj[[i]],]
+      ind <- ind + n
+    }
+  }
+
+  if (deriv==1) { 
+    d1H <- matrix(0,p,m) ## only store diagonals of d1H
+    for (l in 1:m) {
+      for (i in 1:K) {
+        v <- rep(0,n);ind <- 1:n
+        for (q in 1:K) { 
+          v <- v + l3[,i3[i,i,q]] * d1eta[ind,l]
+          ind <- ind + n
+        }
+        d1H[jj[[i]],l] <-  d1H[jj[[i]],l] + colSums(X[,jj[[i]],drop=FALSE]*(v*X[,jj[[i]],drop=FALSE])) 
+      } 
+    }
+  } ## if deriv==1
+
+  if (deriv>1) {
+    d1H <- list()
+    for (l in 1:m) {
+      d1H[[l]] <- matrix(0,p,p)
+      for (i in 1:K) for (j in i:K) {
+        v <- rep(0,n);ind <- 1:n
+        for (q in 1:K) { 
+          v <- v + l3[,i3[i,j,q]] * d1eta[ind,l]
+          ind <- ind + n
+        }
+        ## d1H[[l]][jj[[j]],jj[[i]]] <- 
+        A <- t(X[,jj[[i]],drop=FALSE])%*%(v*X[,jj[[j]],drop=FALSE])
+        d1H[[l]][jj[[i]],jj[[j]]] <- d1H[[l]][jj[[i]],jj[[j]]] + A
+        if (j>i) d1H[[l]][jj[[j]],jj[[i]]] <- d1H[[l]][jj[[j]],jj[[i]]] + t(A)
+      } 
+    }
+  } ## if deriv>1
+
+  if (deriv>2) {
+    ## need tr(Hp^{-1} d^2H/drho_k drho_j)
+    ## First form the expanded model matrix...
+    VX <- Xe <- matrix(0,K*n,ncol(X))
+    ind <- 1:n 
+    for (i in 1:K) { 
+      Xe[ind,jj[[i]]] <- X[,jj[[i]]]
+      ind <- ind + n
+    }
+    ## Now form Hp^{-1} Xe'...
+    if (is.list(fh)) { ## then the supplied factor is an eigen-decomposition
+       d <- fh$values;d[d>0] <- 1/d[d>0];d[d<=0] <- 0
+       Xe <- t(D*((fh$vectors%*%(d*t(fh$vectors)))%*%(D*t(Xe))))
+    } else { ## the supplied factor is a choleski factor
+       ipiv <- piv <- attr(fh,"pivot");ipiv[piv] <- 1:p
+       Xe <- t(D*(backsolve(fh,forwardsolve(t(fh),(D*t(Xe))[piv,]))[ipiv,]))
+    }
+    ## now compute the required trace terms
+    d2eta <- matrix(0,n*K,ncol(d2b))
+    ind <- 1:n
+    for (i in 1:K) { 
+      d2eta[ind,] <- X[,jj[[i]],drop=FALSE]%*%d2b[jj[[i]],]
+      ind <- ind + n
+    }
+    trHid2H <- rep(0,ncol(d2b))
+    kk <- 0 ## counter for second derivatives
+    for (k in 1:m) for (l in k:m) { ## looping over smoothing parameters...
+      kk <- kk + 1
+      for (i in 1:K) for (j in 1:K) {
+        v <- rep(0,n);ind <- 1:n
+        for (q in 1:K) { ## accumulate the diagonal matrix for X_i'diag(v)X_j
+          v <- v + d2eta[ind,kk]*l3[,i3[i,j,q]]
+          ins <- 1:n
+          for (s in 1:K) { 
+            v <- v + d1eta[ind,k]*d1eta[ins,l]*l4[,i4[i,j,q,s]]
+            ins <- ins + n
+          }
+          ind <- ind + n
+        }
+        if (i==j) {
+          rind <- 1:n + (i-1)*n
+          VX[rind,jj[[i]]] <- v * X[,jj[[i]]]
+        } else {
+          rind1 <- 1:n + (i-1)*n
+          rind2 <- 1:n + (j-1)*n
+          VX[rind2,jj[[i]]] <- v * X[,jj[[i]]]
+          VX[rind1,jj[[j]]] <- v * X[,jj[[j]]]
+        }
+      }
+      trHid2H[kk] <- sum(Xe*VX)
+    }
+  } ## if deriv>2
+
   list(lb=lb,lbb=lbb,d1H=d1H,d2H=d2H,trHid2H=trHid2H)
 } ## end of gamlss.gH
 
@@ -485,6 +621,266 @@ gaulss <- function(link=list("identity","logb"),b=0.01) {
 } ## end gaulss
 
 
+multinom <- function(K=1) {
+## general family for multinomial logistic regression model...
+## accepts no links as parameterization directly in terms of 
+## linear predictor. 
+## 1. get derivatives wrt mu, tau
+## 2. get required link derivatives and tri indices.
+## 3. transform derivs to derivs wrt eta (gamlss.etamu).
+## 4. get the grad and Hessian etc for the model
+##    via a call to gamlss.gH  
+## the first derivatives of the log likelihood w.r.t
+## the first and second parameters...
+ 
+  if (K<1) stop("number of categories must be at least 2") 
+   stats <- list()
+  
+  for (i in 1:K) {
+    stats[[i]] <- make.link("identity")
+    fam <- structure(list(link="identity",canonical="none",linkfun=stats[[i]]$linkfun,
+           mu.eta=stats[[i]]$mu.eta),
+           class="family")
+    fam <- fix.family.link(fam)
+    stats[[i]]$d2link <- fam$d2link
+    stats[[i]]$d3link <- fam$d3link
+    stats[[i]]$d4link <- fam$d4link
+  } 
+
+  residuals <- function(object,type=c("deviance")) {
+  ## Deviance residuals where sign depends on whether classification correct (+ve)
+  ## or not (-ve)...
+      type <- match.arg(type)
+      ## get category probabilities...
+      p <- object$family$predict(object$family,eta=object$linear.predictors)[[1]]
+      ## now get most probable category for each observation
+      pc <- apply(p,1,function(x) which(max(x)==x)[1])-1 
+      n <- length(pc)
+      ## +ve sign if class correct, -ve otherwise
+      sgn <- rep(-1,n); sgn[pc==object$y] <- 1
+      ## now get the deviance...
+      sgn*sqrt(-2*log(pmax(.Machine$double.eps,p[1:n + object$y*n]))) 
+  } ## residuals
+
+  predict <- function(family,se=FALSE,eta=NULL,y=NULL,X=NULL,
+                beta=NULL,off=NULL,Vb=NULL) {
+  ## optional function to give predicted values - idea is that 
+  ## predict.gam(...,type="response") will use this, and that
+  ## either eta will be provided, or {X, beta, off, Vb}. family$data
+  ## contains any family specific extra information. 
+  ## if se = FALSE returns one item list containing matrix otherwise 
+  ## list of two matrices "fit" and "se.fit"... 
+
+    if (is.null(eta)) { 
+      lpi <- attr(X,"lpi") 
+      if (is.null(lpi)) {
+        lpi <- list(1:ncol(X))
+      } 
+      K <- length(lpi) ## number of linear predictors
+      eta <- matrix(0,nrow(X),K)
+      if (se) { 
+        ve <- matrix(0,nrow(X),K) ## variance of eta
+        ce <- matrix(0,nrow(X),K*(K-1)/2) ## covariance of eta_i eta_j
+      } 
+      for (i in 1:K) { 
+        Xi <- X[,lpi[[i]],drop=FALSE]
+        eta[,i] <- Xi%*%beta[lpi[[i]]] ## ith linear predictor
+        if (se) { ## variance and covariances for kth l.p.
+          ve[,i] <- drop(pmax(0,rowSums((Xi%*%Vb[lpi[[i]],lpi[[i]]])*Xi)))
+          ii <- 0
+          if (i<K) for (j in (i+1):K) {
+            ii <- ii + 1
+            ce[,ii] <- drop(pmax(0,rowSums((Xi%*%Vb[lpi[[i]],lpi[[j]]])*X[,lpi[[j]]])))
+          }
+        }
+      }
+    } else { 
+      se <- FALSE
+    }
+    gamma <- cbind(1,exp(eta))
+    beta <- rowSums(gamma)
+    gamma <- gamma/beta ## category probabilities
+    vp <- gamma*0
+    if (se) { ## need to loop to find se of probabilities...
+      for (j in 1:(K+1)) {
+        ## get dp_j/deta_k...
+        if (j==1) dp <- -gamma[,-1,drop=FALSE]/beta else { 
+          dp <- -gamma[,j]*gamma[,-1,drop=FALSE]
+          dp[,j-1] <- gamma[,j]*(1-gamma[,j]) 
+        }
+        ## now compute variance... 
+        vp[,j] <- rowSums(dp^2*ve)
+        ii <- 0
+        for (i in 1:K) if (i<K) for (k in (i+1):K) {
+          ii <- ii + 1
+          vp[,j] <- vp[,j] + 2 * dp[,i]*dp[,k]*ce[,ii] 
+        }
+        vp[,j] <- sqrt(pmax(0,vp[,j])) ## transform to se
+      }
+      return(list(fit=gamma,se.fit=vp))
+    } ## if se
+    list(fit=gamma)
+  } ## multinom predict
+
+  postproc <- expression({
+    ## code to evaluate in estimate.gam, to evaluate null deviance
+    multinom <- list()
+    object$y <- round(object$y) 
+    multinom$nj <- tabulate(object$y+1) ## count each class membership
+    multinom$n <- sum(multinom$nj) ## total number
+    multinom$K <- length(multinom$nj)-1 ## number of linear predictors
+    ## compute exp(eta) for categories 1..K
+    multinom$gamma <- c(1,solve(diag(multinom$n/multinom$nj[-1],multinom$K)-
+                        matrix(1,multinom$K,multinom$K),rep(1,multinom$K)))
+    multinom$gamma <- log(multinom$gamma/sum(multinom$gamma))
+    object$null.deviance <- -2*sum(multinom$gamma[object$y+1])
+  })
+
+  ll <- function(y,X,coef,wt,family,deriv=0,d1b=0,d2b=0,Hp=NULL,rank=0,fh=NULL,D=NULL,eta=NULL) {
+  ## Function defining the logistic multimomial model log lik. 
+  ## Assumption is that coding runs from 0..K, with 0 class having no l.p.
+  ## argument eta is for debugging only, and allows direct FD testing of the 
+  ## derivatives w.r.t. eta. 
+  ## ... this matches binary log reg case... 
+  ## deriv: 0 - eval
+  ##        1 - grad and Hess
+  ##        2 - diagonal of first deriv of Hess
+  ##        3 - first deriv of Hess
+  ##        4 - everything.
+    n <- length(y)
+    if (is.null(eta)) {
+      return.l <- FALSE
+      jj <- attr(X,"lpi") ## extract linear predictor index
+      K <- length(jj) ## number of linear predictors 
+      eta <- matrix(1,n,K+1) ## linear predictor matrix (dummy 1's in first column)
+      for (i in 1:K) eta[,i+1] <- X[,jj[[i]],drop=FALSE]%*%coef[jj[[i]]]
+    } else { l2 <- 0;K <- ncol(eta);eta <- cbind(1,eta); return.l <- TRUE}
+ 
+    if (K!=family$nlp) stop("number of linear predictors doesn't match")
+    y <- round(y) ## just in case
+    if (min(y)<0||max(y)>K) stop("response not in 0 to number of predictors + 1")
+    
+    ee <- exp(eta[,-1,drop=FALSE])
+    beta <- 1 + rowSums(ee); alpha <- log(beta)
+    
+    l0 <- eta[1:n+y*n] - alpha ## log likelihood
+    l <- sum(l0)    
+
+    l1 <- matrix(0,n,K) ## first deriv matrix
+ 
+    if (deriv>0) {
+      for (i in 1:K) l1[,i] <- ee[,i]/beta ## alpha1
+    
+      ## the second derivatives...
+    
+      l2 <- matrix(0,n,K*(K+1)/2)
+      ii <- 0; b2 <- beta^2
+      for (i in 1:K) for (j in i:K) {
+        ii <- ii + 1 ## column index
+        l2[,ii] <- if (i==j) -l1[,i] + ee[,i]^2/b2 else (ee[,i]*ee[,j])/b2
+      }
+
+      ## finish first derivatives...
+      for (i in 1:K) l1[,i] <- as.numeric(y==i) - l1[,i] 
+
+    } ## if (deriv>0)
+ 
+    l3 <- l4 <- 0 ## defaults
+    tri <- family$tri ## indices to facilitate access to earlier results
+    
+    if (deriv>1) { ## the third derivatives...
+      l3 <- matrix(0,n,(K*(K+3)+2)*K/6)
+      ii <- 0; b3 <- b2 * beta
+      for (i in 1:K) for (j in i:K) for (k in j:K) {
+        ii <- ii + 1 ## column index
+        if (i==j&&j==k) { ## all same
+           l3[,ii] <- l2[,tri$i2[i,i]] + 2*ee[,i]^2/b2 - 2*ee[,i]^3/b3
+        } else if (i!=j&&j!=k&i!=k) { ## all different
+           l3[,ii] <- -2*(ee[,i]*ee[,j]*ee[,k])/b3
+        } else { ## two same one different
+           kk <- if (i==j) k else j ## get indices for differing pair
+           l3[,ii] <- l2[,tri$i2[i,kk]] - 2*(ee[,i]*ee[,j]*ee[,k])/b3
+        }
+      }
+    } ## if (deriv>1)
+
+    if (deriv>3) { ## the fourth derivatives...
+      l4 <- matrix(0,n,(6+K*11+K^2*6+K^3)*K/24)
+      ii <- 0; b4 <- b3 * beta
+      for (i in 1:K) for (j in i:K) for (k in j:K) for (l in k:K) {
+        ii <- ii + 1 ## column index
+        uni <- unique(c(i,j,k,l));
+        nun <- length(uni) ## number of unique indices
+        if (nun==1) { ## all equal
+          l4[,ii] <- l3[,tri$i3[i,i,i]] + 4*ee[,i]^2/b2 - 10*ee[,i]^3/b3 + 6*ee[,i]^4/b4
+        } else if (nun==4) { ## all unequal
+          l4[,ii] <- 6*ee[,i]*ee[,j]*ee[,k]*ee[,l]/b4
+        } else if (nun==3) { ## 2 same 2 different
+          l4[,ii] <- l3[,tri$i3[uni[1],uni[2],uni[3]]]  +6*ee[,i]*ee[,j]*ee[,k]*ee[,l]/b4
+        } else if (sum(uni[1]==c(i,j,k,l))==2) { ## 2 unique (2 of each)
+          l4[,ii] <- l3[,tri$i3[uni[1],uni[2],uni[2]]] - 2 * ee[,uni[1]]^2*ee[,uni[2]]/b3 + 
+                     6*ee[,i]*ee[,j]*ee[,k]*ee[,l]/b4
+        } else { ## 3 of one 1 of the other
+          if (sum(uni[1]==c(i,j,k,l))==1) uni <- uni[2:1] ## first index is triple repeat index
+          l4[,ii] <- l3[,tri$i3[uni[1],uni[1],uni[2]]] - 4 * ee[,uni[1]]^2*ee[,uni[2]]/b3 + 
+                     6*ee[,i]*ee[,j]*ee[,k]*ee[,l]/b4
+        }
+      }
+    } ## if deriv>3
+
+    if (return.l) return(list(l=l0,l1=l1,l2=l2,l3=l3,l4=l4)) ## for testing...
+
+    if (deriv) {
+      ## get the gradient and Hessian...
+      ret <- gamlss.gH(X,jj,l1,l2,tri$i2,l3=l3,i3=tri$i3,l4=l4,i4=tri$i4,
+                      d1b=d1b,d2b=d2b,deriv=deriv-1,fh=fh,D=D) 
+    } else ret <- list()
+    ret$l <- l; ret
+  } ## end ll multinom
+
+  rd <- function(mu,wt,scale) {
+    ## simulate data given fitted linear predictor matrix in mu 
+    p <- exp(cbind(0,mu))
+    p <- p/rowSums(p)
+    cp <- t(apply(p,1,cumsum))
+    apply(cp,1,function(x) min(which(x>runif(1))))-1    
+  } ## rd
+
+  initialize <- expression({
+  ## Binarize each category and lm on 6*y-3 by category.
+ 
+      n <- rep(1, nobs)
+      ## should E be used unscaled or not?..
+      use.unscaled <- if (!is.null(attr(E,"use.unscaled"))) TRUE else FALSE
+      if (is.null(start)) {
+        jj <- attr(x,"lpi")
+        start <- rep(0,ncol(x))
+        for (k in 1:length(jj)) { ## loop over the linear predictors      
+          yt1 <- 6*as.numeric(y==k)-3
+          x1 <- x[,jj[[k]],drop=FALSE]
+          e1 <- E[,jj[[k]],drop=FALSE] ## square root of total penalty
+          if (use.unscaled) {
+            qrx <- qr(rbind(x1,e1))
+            x1 <- rbind(x1,e1)
+            startji <- qr.coef(qr(x1),c(yt1,rep(0,nrow(E))))
+            startji[!is.finite(startji)] <- 0       
+          } else startji <- pen.reg(x1,e1,yt1)
+          start[jj[[k]]] <- startji ## copy coefficients back into overall start coef vector
+        } ## lp loop
+      }
+  }) ## initialize multinom
+
+  structure(list(family="multinom",ll=ll,link=NULL,#paste(link),
+    nlp=round(K),rd=rd,
+    tri = trind.generator(K), ## symmetric indices for accessing derivative arrays
+    initialize=initialize,postproc=postproc,residuals=residuals,predict=predict,
+    linfo = stats, ## link information list
+    d2link=1,d3link=1,d4link=1, ## signals to fix.family.link that all done    
+    ls=1, ## signals that ls not needed here
+    available.derivs = 2 ## can use full Newton here
+    ),class = c("general.family","extended.family","family"))
+} ## end multinom
+
 
 
 pen.reg <- function(x,e,y) {
@@ -594,6 +990,7 @@ ldg <- function(g,deriv=4) {
   ## so l'' = alpha*(b-eg)...
   b <- egi*(1+egi/6)/2
   l2[ind] <- a[ind]*(b-egi)
+  l2[ii] <- -exp(g[ii])
   l3 <- l4 <- NULL
   ## in a similar vein l3 can be robustified...  
   if (deriv>1) {
@@ -609,7 +1006,13 @@ ldg <- function(g,deriv=4) {
    
     l4[ii] <- -exp(g[ii])
   }
-  list(l1=-a,l2=l2,l3=l3,l4=l4)
+  l1=-a
+  ghi <- log(.Machine$double.xmax)/5
+  ii <- g > ghi
+  if (sum(ii)) {
+    l1[ii] <- l2[ii] <- l3[ii] <- l4[ii] <- -exp(ghi)
+  }
+  list(l1=l1,l2=l2,l3=l3,l4=l4)
 } ## ldg
 
 lde <- function(eta,deriv=4) {
@@ -696,8 +1099,8 @@ zipll <- function(y,g,eta,deriv=0) {
       ## order gggg,ggge,ggee,geee,eeee
       l4 <- matrix(0,n,5) 
       l4[!zind,1] <- lg$l4[!zind]   ## l_gggg, y>0
-      l4[!zind,4] <- le$l4[!zind]   ## l_eeee, y>0
-      l4[zind,4]  <- l[zind]        ## l_eeee, y=0
+      l4[!zind,5] <- le$l4[!zind]   ## l_eeee, y>0
+      l4[zind,5]  <- l[zind]        ## l_eeee, y=0
    }
    list(l=l,l1=l1,l2=l2,l3=l3,l4=l4,El2=El2)
 } ## zipll
@@ -764,6 +1167,8 @@ ziplss <-  function(link=list("identity","identity")) {
   ## predict.gam(...,type="response") will use this, and that
   ## either eta will be provided, or {X, beta, off, Vb}. family$data
   ## contains any family specific extra information. 
+  ## if se = FALSE returns one item list containing matrix otherwise 
+  ## list of two matrices "fit" and "se.fit"... 
 
     if (is.null(eta)) { 
       lpi <- attr(X,"lpi") 
