@@ -50,8 +50,6 @@ struct mrec
 };
 typedef struct mrec MREC;
 
-void ErrorMessage(char *msg,int fatal);
-
 matrix null_mat;
 MREC *top,*bottom;
 
@@ -64,22 +62,22 @@ matrix initmat(int rows,int cols)
   pad=0;
 #endif
   A.vec=0;
-  A.M=(double **)R_chk_calloc((size_t)(rows+2*pad),sizeof(double *));
+  A.M=(double **)CALLOC((size_t)(rows+2*pad),sizeof(double *));
   if ((cols==1)||(rows==1))
   { if (A.M)
-    A.M[0]=(double *)R_chk_calloc((size_t)(cols*rows+2*pad),sizeof(double));
+    A.M[0]=(double *)CALLOC((size_t)(cols*rows+2*pad),sizeof(double));
     for (i=1;i<rows+2*pad;i++)
     A.M[i]=A.M[0]+i*cols;A.vec=1;
   } else
   { if (A.M)
     for (i=0;i<rows+2*pad;i++)
-    A.M[i]=(double *)R_chk_calloc((size_t)(cols+2*pad),sizeof(double));
+    A.M[i]=(double *)CALLOC((size_t)(cols+2*pad),sizeof(double));
   }
   A.mem=(long)(rows*cols*sizeof(double));
   memused+=A.mem;matrallocd++;
   A.original_r=A.r=rows;A.original_c=A.c=cols;
   if (((!A.M)||(!A.M[rows-1+2*pad]))&&(rows*cols>0))
-  { ErrorMessage(_("Failed to initialize memory for matrix."),1);}
+  { error(_("Failed to initialize memory for matrix."));}
   if (pad)  /* This lot is debugging code that checks out matrix errors
 		       on allocation and release */
   { if (A.vec)
@@ -100,10 +98,10 @@ matrix initmat(int rows,int cols)
     A.V=A.M[0];
   /* putting a record of the matrix on the linked list of all extant matrices */
     if (matrallocd==1) /*new list*/
-    { top=bottom=(MREC *)R_chk_calloc(1,sizeof(MREC));
+    { top=bottom=(MREC *)CALLOC(1,sizeof(MREC));
       bottom->mat=top->mat=A;top->bp=bottom;bottom->fp=top;
     } else  /* expanding the linked list by one */
-    { top->fp=(MREC *)R_chk_calloc(1,sizeof(MREC));
+    { top->fp=(MREC *)CALLOC(1,sizeof(MREC));
       top->fp->mat=A;top->fp->bp=top;top=top->fp; /* crystal clear, no? */
     }
   }
@@ -141,13 +139,13 @@ void freemat(matrix A)
 	     }
       }
       if (!ok)
-      { ErrorMessage(_("An out of bound write to matrix has occurred!"),1);
+      { error(_("An out of bound write to matrix has occurred!"),1);
       }
       /* find the matrix being deleted in the linked list of extant matrices */
       i=0;delet=bottom;
       while ((i<matrallocd)&&(delet->mat.M!=A.M)) { i++;delet=delet->fp;}
       if (i==matrallocd)
-      { ErrorMessage(_("INTEGRITY PROBLEM in the extant matrix list."),1);
+      { error(_("INTEGRITY PROBLEM in the extant matrix list."));
       } else
       { if (i)
 	     delet->bp->fp=delet->fp;
@@ -155,16 +153,16 @@ void freemat(matrix A)
 	     if (i!=matrallocd-1)
 	     delet->fp->bp=delet->bp;
 	     else top=delet->bp;
-	     R_chk_free(delet);
+	     FREE(delet);
       }
       /* repositioning pointers so that what was allocated gets freed */
       if (!A.vec) for (i=0;i<pad;i++) A.M--;
       for (i=0;i<A.original_r+2*pad;i++)
       for (j=0;j<pad;j++) A.M[i]--;
     }
-    if (A.vec) R_chk_free(A.M[0]); else
-    for (i=0;i<A.original_r+2*pad;i++) if (A.M[i]) R_chk_free(A.M[i]);
-    if (A.M) R_chk_free(A.M);
+    if (A.vec) FREE(A.M[0]); else
+    for (i=0;i<A.original_r+2*pad;i++) if (A.M[i]) FREE(A.M[i]);
+    if (A.M) FREE(A.M);
     memused -= A.mem;matrallocd--;
   }
 }
@@ -179,7 +177,7 @@ void matrixintegritycheck()
   int ok=1,pad=PAD,i,j,k=0;
   matrix A;
 #ifndef RANGECHECK
-  ErrorMessage(_("You are trying to check matrix integrity without defining RANGECHECK."));
+  error(_("You are trying to check matrix integrity without defining RANGECHECK."));
 #endif
   B=bottom;
   while (k<matrallocd)
@@ -203,7 +201,7 @@ void matrixintegritycheck()
       }
     }
     if (!ok)
-    { ErrorMessage(_("An out of bound write to matrix has occurred!"),1);
+    { error(_("An out of bound write to matrix has occurred!"));
     }
     k++;B=B->fp;
   }
@@ -241,7 +239,7 @@ void mcopy(matrix *A,matrix *B)
 
 { int Ac;
   double *pA,*pB,**AM,**BM;
-  if (A->r>B->r||A->c>B->c) ErrorMessage(_("Target matrix too small in mcopy"),1);
+  if (A->r>B->r||A->c>B->c) error(_("Target matrix too small in mcopy"));
   BM=B->M;Ac=A->c;
   for (AM=A->M;AM<A->M+A->r;AM++)
   { pB= *BM;
@@ -261,7 +259,7 @@ void matmult(C,A,B,tA,tB) matrix C,A,B;int tA,tB;
   if (tA)
   { if (tB)
     { if ((A.r!=B.c)||(A.c!=C.r)||(B.r!=C.c))
-      { ErrorMessage(_("Incompatible matrices in matmult."),1);}
+      { error(_("Incompatible matrices in matmult."));}
       for (i=0;i<A.c;i++) for (j=0;j<B.r;j++)
       { p2=CM[i]+j;(*p2)=0.0;p=BM[j];
 	for (k=0;k<A.r;k++)
@@ -269,7 +267,7 @@ void matmult(C,A,B,tA,tB) matrix C,A,B;int tA,tB;
       }
     } else
     { if ((A.r!=B.r)||(A.c!=C.r)||(B.c!=C.c))
-      { ErrorMessage(_("Incompatible matrices in matmult."),1);}
+      { error(_("Incompatible matrices in matmult."));}
       for (i=0;i<A.c;i++)
       for (p=CM[i];p<(CM[i]+C.c);p++)
       (*p)=0.0;
@@ -282,7 +280,7 @@ void matmult(C,A,B,tA,tB) matrix C,A,B;int tA,tB;
   } else
   { if (tB)
     { if ((A.c!=B.c)||(A.r!=C.r)||(B.r!=C.c))
-      { ErrorMessage(_("Incompatible matrices in matmult."),1);}
+      { error(_("Incompatible matrices in matmult."));}
       for (i=0;i<A.r;i++) for (j=0;j<B.r;j++)
       { p2=CM[i]+j;*p2=0.0;p1=BM[j];
 	for (p=AM[i];p<(AM[i]+A.c);p++)
@@ -290,7 +288,7 @@ void matmult(C,A,B,tA,tB) matrix C,A,B;int tA,tB;
       }
     } else
     { if ((A.c!=B.r)||(C.r!=A.r)||(C.c!=B.c))
-      { ErrorMessage(_("Incompatible matrices in matmult."),1);}
+      { error(_("Incompatible matrices in matmult."));}
       for (i=0;i<A.r;i++) for (p=CM[i];p<(CM[i]+B.c);p++) *p=0.0;
       for (k=0;k<A.c;k++) for (i=0;i<A.r;i++)
       { p1=BM[k];temp=AM[i][k];
@@ -341,8 +339,8 @@ void multi(int n,matrix C,...)
 #else
   va_start(argptr,C);
 #endif
-  t=(int *)R_chk_calloc((size_t)n,sizeof(int));
-  M=(matrix *)R_chk_calloc((size_t)n,sizeof(matrix)); /* previously no cast for sun C ??! */
+  t=(int *)CALLOC((size_t)n,sizeof(int));
+  M=(matrix *)CALLOC((size_t)n,sizeof(matrix)); /* previously no cast for sun C ??! */
   for (i=0;i<n;i++) M[i]=/*(matrix)*/ va_arg(argptr,matrix);
   for (i=0;i<n;i++) t[i]=(int) va_arg(argptr,int);
   if (t[0]) r=M[0].c; else r=M[0].r;
@@ -360,8 +358,8 @@ void multi(int n,matrix C,...)
   { matmult(C,temp0,M[n-1],0,t[n-1]);
     freemat(temp0);
   }
-  R_chk_free(t);
-  R_chk_free(M);
+  FREE(t);
+  FREE(M);
   va_end(argptr);
 }
 
@@ -377,11 +375,11 @@ void invert(matrix *A)
 
 { double **AM,*p,*p1,max,x;
   int *c,*rp,*cp,i,j,k,pr=0,pc=0,*d,cj,ck;
-  if (A->r!=A->c) ErrorMessage(_("Attempt to invert() non-square matrix"),1);
-  c=(int *)R_chk_calloc((size_t)A->c,sizeof(int)); /* index of columns, used for column pivoting */
-  d=(int *)R_chk_calloc((size_t)A->c,sizeof(int));
-  rp=(int *)R_chk_calloc((size_t)A->c,sizeof(int)); /* row changes */
-  cp=(int *)R_chk_calloc((size_t)A->c,sizeof(int)); /* row changes */
+  if (A->r!=A->c) error(_("Attempt to invert() non-square matrix"));
+  c=(int *)CALLOC((size_t)A->c,sizeof(int)); /* index of columns, used for column pivoting */
+  d=(int *)CALLOC((size_t)A->c,sizeof(int));
+  rp=(int *)CALLOC((size_t)A->c,sizeof(int)); /* row changes */
+  cp=(int *)CALLOC((size_t)A->c,sizeof(int)); /* row changes */
   for (i=0;i<A->c;i++) { c[i]=i;d[i]=i;}
   AM=A->M;            /* saving adress calculations*/
   for (j=0;j<A->c;j++) /* loop through columns to be reduced */
@@ -399,7 +397,7 @@ void invert(matrix *A)
     cj=c[j]; /* save time */
     /* Now reduce the column */
     x=AM[j][cj];
-    if (x==0.0) ErrorMessage(_("Singular Matrix passed to invert()"),1);
+    if (x==0.0) error(_("Singular Matrix passed to invert()"));
     for (p=AM[j];p<AM[j]+A->c;p++) *p/=x; /* divide row j by pivot element */
     AM[j][cj]=1.0/x;
     for (i=0;i<A->r;i++) /* work down rows eliminating column j */
@@ -436,7 +434,7 @@ void invert(matrix *A)
     { p=AM[k];x=p[i];p[i]=p[rp[i]];p[rp[i]]=x;} /* column exchange  */
   }
    
-  R_chk_free(c);R_chk_free(rp);R_chk_free(cp);R_chk_free(d);
+  FREE(c);FREE(rp);FREE(cp);FREE(d);
 }
 
 void tricholeski(matrix *T,matrix *l0,matrix *l1)
@@ -806,7 +804,7 @@ int QR(matrix *Q,matrix *R)
   double *u,t,z,**RM,*p,m;
   RM=R->M;Rr=R->r;
   if (Rr<R->c) n=Rr; else n=R->c;
-  u=(double *)R_chk_calloc((size_t)Rr,sizeof(double));
+  u=(double *)CALLOC((size_t)Rr,sizeof(double));
   for (k=0;k<n;k++)
   { m=0.0;for (i=k;i<Rr;i++) { z=RM[i][k];z=fabs(z);if (z>m) m=z;}
     if (m) for (i=k;i<Rr;i++) RM[i][k]/=m; /* avoid over/underflow problems */
@@ -818,7 +816,7 @@ int QR(matrix *Q,matrix *R)
     t=t*t;t+=u[k]*u[k]-z*z; /* efficient t calculation */
     /* t=0.0;for (p=u+k;p<u+Rr;p++) t+= *p * *p; - old inefficient calculation */
     t=sqrt(t/2);
-    if (t==0.0) {R_chk_free(u);return(0);} /* singular matrix */
+    if (t==0.0) {FREE(u);return(0);} /* singular matrix */
     for (p=u+k;p<u+Rr;p++) *p /= t;
     for (j=k+1;j<R->c;j++)
     { t=0.0;for (i=k;i<Rr;i++) t+=u[i]*RM[i][j];
@@ -829,7 +827,7 @@ int QR(matrix *Q,matrix *R)
       for (i=k;i<Rr;i++) p[i]=u[i];
     }
   }
-  R_chk_free(u);
+  FREE(u);
   return(1);
 }
 
@@ -950,8 +948,7 @@ void root(matrix *M,matrix *C,double tol)
   }
   if (m>10.0*DOUBLE_EPS*max) ok=0;
   if (!ok)
-  { /*ErrorMessage("Using svd to find root of penalty!",0);*/ 
-    (*C)=svdroot(*M,tol);
+  { (*C)=svdroot(*M,tol);
     freemat(U);freemat(T);freemat(u0);freemat(u1);
     return; 
   }
@@ -1320,7 +1317,7 @@ void svd_bidiag(matrix *U, matrix *w, matrix *ws,matrix *V)
       }
     }  
     if (k==maxreps) 
-    ErrorMessage(_("svd() not converged"),1);
+    error(_("svd() not converged"));
   }
   /* make all singular values  non-negative */
   for (i=0;i<w->r;i++) 
@@ -1397,7 +1394,7 @@ matrix svdroot(matrix A,double reltol)
       prod=0.0;for (j=0;j<a.r;j++) prod+=a.M[j][i]*v.M[j][i];
       if (prod<0.0) 
 	  { sprintf(err,_("svdroot matrix not +ve semi def. %g"),w.V[i]*w.V[i]);
-		ErrorMessage(err,1); 
+		error(err); 
 	  }
     }
   }
@@ -1424,7 +1421,7 @@ void sort(matrix a)
 { int i;
   qsort(a.V,(size_t)a.r*a.c,sizeof(a.V[0]),elemcmp);
   for (i=0;i<a.r*a.c-1;i++) if (a.V[i]>a.V[i+1])
-  ErrorMessage(_("Sort failed"),1);
+  error(_("Sort failed"));
 
 }
 
